@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_esri_io.c 12291 2013-10-08 02:33:12Z pwessel $
+ *	$Id: gmt_esri_io.c 13846 2014-12-28 21:46:54Z pwessel $
  *
- *	Copyright (c) 1991-2013 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2015 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -43,7 +43,9 @@ int GMT_is_esri_grid (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header) {
 	if ((fp = GMT_fopen (GMT, header->name, "r")) == NULL)
 		return (GMT_GRDIO_OPEN_FAILED);
 
-	(void)fgets (record, GMT_BUFSIZ, fp);	/* Just get first line. Not using GMT_fgets since we may be reading a binary file */
+	if (fgets (record, GMT_BUFSIZ, fp) == NULL) {	/* Just get first line. Not using GMT_fgets since we may be reading a binary file */
+		return (GMT_GRDIO_OPEN_FAILED);
+	}
 	GMT_fclose (GMT, fp);
 	if (strncmp (record, "ncols ", 6) ) {
 		/* Failed to find "ncols"; probably a binary file */
