@@ -1,4 +1,4 @@
-/* $Id: psldemo.c 12822 2014-01-31 23:39:56Z remko $
+/* $Id: psldemo.c 13485 2014-08-31 00:55:53Z pwessel $
  *
  * psldemo.c: Test all PSL functions at least once
  */
@@ -16,7 +16,7 @@
 
 int main (int argc, char **argv) {
 	struct PSL_CTRL *PSL = NULL;
-	double Letter[2] = {612.0, 792.0}, size[7];
+	double Letter[2] = {612.0, 792.0}, size[PSL_MAX_DIMS];
 	double rgb[8][4] = {{1.0, 0.0, 0.0, 0.0}, {0.5, 0.7, 0.1, 0.0}, {-1.0, -1.0, -1.0, 0.0}, {0.0, 0.0, 0.0, 0.0}, {1.0, 1.0, 1.0, 0.0},
 		{-3.0, -3.0, -3.0, 0.0}, {0.9, 0.9, 9.0, 0.0}, {0.5, 0.7, 0.1, 0.5}};
 	double offset[2] = {0.1, 0.1}, scales[2] = {1.0, 1.0};
@@ -33,7 +33,10 @@ that can be used to create plots.  The resulting @%6%PostScript@%% code is ASCII
 	struct imageinfo h;
 	unsigned char *picture;
 
+	if (argc > 1) fprintf (stderr, "psldemo: No argument required\n");
+	
 	memset (&h, 0, sizeof(struct imageinfo)); /* initialize struct */
+	memset (size, 0, PSL_MAX_DIMS*sizeof(double)); /* initialize array */
 
 	size[0] = 0.3;
 
