@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pslegend.c 12915 2014-02-18 17:26:46Z pwessel $
+ *	$Id: pslegend.c 14150 2015-03-06 00:33:36Z pwessel $
  *
- *	Copyright (c) 1991-2014 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2015 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -150,7 +150,7 @@ int GMT_pslegend_parse (struct GMT_CTRL *GMT, struct PSLEGEND_CTRL *Ctrl, struct
 		switch (opt->option) {
 
 			case '<':	/* Input files */
-				if (!GMT_check_filearg (GMT, '<', opt->arg, GMT_IN)) n_errors++;
+				if (!GMT_check_filearg (GMT, '<', opt->arg, GMT_IN, GMT_IS_TEXTSET)) n_errors++;
 				break;
 
 			/* Processes program-specific parameters */
@@ -297,7 +297,7 @@ void drawbase (struct GMT_CTRL *GMT, struct PSL_CTRL *PSL, double x0, double x1,
 }
 #endif
 
-struct GMT_TEXTSET *alloc_if_not_done_already (struct GMTAPI_CTRL *API, struct GMT_TEXTSET *Din, unsigned int geometry)
+struct GMT_TEXTSET *get_textset_pointer (struct GMTAPI_CTRL *API, struct GMT_TEXTSET *Din, unsigned int geometry)
 {
 	uint64_t dim[3] = {1, 1, GMT_SMALL_CHUNK};
 	struct GMT_TEXTSET *D = NULL;
@@ -309,6 +309,18 @@ struct GMT_TEXTSET *alloc_if_not_done_already (struct GMTAPI_CTRL *API, struct G
 	return (D);
 }
 
+struct GMT_DATASET *get_dataset_pointer (struct GMTAPI_CTRL *API, struct GMT_DATASET *Din, unsigned int geometry)
+{
+	uint64_t dim[4] = {1, GMT_SMALL_CHUNK, 2, 2};	/* We will a 2-row data set for up to 64 lines; allocate just once */
+	struct GMT_DATASET *D = NULL;
+	if (Din) return Din;	/* Already done this */
+	if (D == NULL && (D = GMT_Create_Data (API, GMT_IS_DATASET, geometry, 0, dim, NULL, NULL, 0, 0, NULL)) == NULL) {
+		GMT_Report (API, GMT_MSG_NORMAL, "Unable to create a data set for pslegend\n");
+		return (NULL);
+	}
+	return (D);
+}
+
 /* Define the fraction of the height of the font to the font size */
 #define FONT_HEIGHT_PRIMARY (GMT->session.font[GMT->current.setting.font_annot[0].id].height)
 #define FONT_HEIGHT(font_id) (GMT->session.font[font_id].height)
@@ -317,7 +329,12 @@ struct GMT_TEXTSET *alloc_if_not_done_already (struct GMTAPI_CTRL *API, struct G
 #define SYM 0
 #define TXT 1
 #define PAR 2
-#define N_CMD 3
+#define N_TXT 3
+#define FRONT 0
+#define QLINE 1
+#define N_DAT 2
+
+int debug_print = 0;	/* Change to 1 to get debug print for some buffer strings */
 
 int GMT_pslegend (void *V_API, int mode, void *args)
 {	/* High-level function that implements the pslegend task */
@@ -325,7 +342,7 @@ int GMT_pslegend (void *V_API, int mode, void *args)
 	int i, k, n = 0, justify = 0, n_columns = 1, error = 0, column_number = 0, id, n_scan;
 	int status = 0, object_ID;
 	bool flush_paragraph = false, draw_vertical_line = false, gave_label, gave_mapscale_options, did_old = false;
-	uint64_t seg, row;
+	uint64_t seg, row, n_fronts = 0, n_quoted_lines = 0;
 	size_t n_char = 0;
 	 
 	char txt_a[GMT_LEN256] = {""}, txt_b[GMT_LEN256] = {""}, txt_c[GMT_LEN256] = {""}, txt_d[GMT_LEN256] = {""};
@@ -335,14 +352,13 @@ int GMT_pslegend (void *V_API, int mode, void *args)
 	char font[GMT_LEN256] = {""}, lspace[GMT_LEN256] = {""}, tw[GMT_LEN256] = {""}, jj[GMT_LEN256] = {""};
 	char bar_cpt[GMT_LEN256] = {""}, bar_gap[GMT_LEN256] = {""}, bar_height[GMT_LEN256] = {""}, bar_opts[GMT_BUFSIZ] = {""};
 	char *opt = NULL, sarg[GMT_LEN256] = {""}, txtcolor[GMT_LEN256] = {""}, buffer[GMT_BUFSIZ] = {""}, A[GMT_LEN32] = {""};
-	char B[GMT_LEN32] = {""}, C[GMT_LEN32] = {""};
+	char path[GMT_BUFSIZ] = {""}, B[GMT_LEN32] = {""}, C[GMT_LEN32] = {""};
 	char *line = NULL, string[GMT_STR16] = {""}, save_EOF = 0, *c = NULL;
 #ifdef DEBUG
 	int guide = 0;
 #endif
 
 	unsigned char *dummy = NULL;
-	
 
 	double x_orig, y_orig, x_off, x, y, r, x0, y0, dx, dy, L, off_ss, off_tt, V = 0.0, sdim[3] = {0.0, 0.0, 0.0};
 	double half_line_spacing, quarter_line_spacing, one_line_spacing, y_start = 0.0, d_off, height, az1, az2, m_az;
@@ -355,10 +371,10 @@ int GMT_pslegend (void *V_API, int mode, void *args)
 	struct GMT_OPTION *r_ptr = NULL, *j_ptr = NULL;
 	struct GMT_FONT ifont;
 	struct GMT_PEN current_pen;
-	struct GMT_TEXTSET *In = NULL, *D[N_CMD] = {NULL, NULL, NULL};
-	struct GMT_DATASET *Front = NULL;
-	struct GMT_TEXTSEGMENT *S[N_CMD] = {NULL, NULL, NULL};
-	struct GMT_DATASEGMENT *F = NULL;
+	struct GMT_TEXTSET *In = NULL, *T[N_TXT];
+	struct GMT_TEXTSEGMENT *Ts[N_TXT];
+	struct GMT_DATASET *D[N_DAT];
+	struct GMT_DATASEGMENT *Ds[N_DAT];
 	struct GMTAPI_CTRL *API = GMT_get_API_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
 
 	memset (&header, 0, sizeof(struct imageinfo)); /* initialize struct */
@@ -381,6 +397,11 @@ int GMT_pslegend (void *V_API, int mode, void *args)
 
 	/*---------------------------- This is the pslegend main code ----------------------------*/
 
+	GMT_memset (T, N_TXT, struct GMT_TEXTSET *);		/* Set these arrays to NULL */
+	GMT_memset (Ts, N_TXT, struct GMT_TEXTSEGMENT *);
+	GMT_memset (D, N_DAT, struct GMT_DATASET *);		/* Set these arrays to NULL */
+	GMT_memset (Ds, N_DAT, struct GMT_DATASEGMENT *);
+	
 	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input text table data\n");
 	if (GMT_compat_check (GMT, 4)) {
 		/* Since pslegend v4 used '>' to indicate a paragraph record we avoid confusion with multiple segmentheaders by *
@@ -639,6 +660,7 @@ int GMT_pslegend (void *V_API, int mode, void *args)
 	if (guide) drawbase (GMT, PSL, Ctrl->D.lon, Ctrl->D.lon + Ctrl->D.width, y0);
 #endif
 
+	flush_paragraph = false;
 	for (tbl = 0; tbl < In->n_tables; tbl++) {	/* We only expect one table but who knows what the user does */
 		for (seg = 0; seg < In->table[tbl]->n_segments; seg++) {	/* We only expect one segment in each table but again... */
 			for (row = 0; row < In->table[tbl]->segment[seg]->n_rows; row++) {	/* Finally processing the rows */
@@ -696,7 +718,7 @@ int GMT_pslegend (void *V_API, int mode, void *args)
 						break;
 
 					case 'H':	/* Header record */
-						if ((D[TXT] = alloc_if_not_done_already (API, D[TXT], GMT_IS_NONE)) == NULL) return (API->error);
+						if ((T[TXT] = get_textset_pointer (API, T[TXT], GMT_IS_NONE)) == NULL) return (API->error);
 						sscanf (&line[2], "%s %s %[^\n]", size, font, text);
 						if (size[0] == '-') size[0] = 0;
 						if (font[0] == '-') font[0] = 0;
@@ -706,10 +728,10 @@ int GMT_pslegend (void *V_API, int mode, void *args)
 						d_off = 0.5 * (Ctrl->L.spacing - FONT_HEIGHT (ifont.id)) * ifont.size / PSL_POINTS_PER_INCH;	/* To center the text */
 						y0 -= Ctrl->L.spacing * ifont.size / PSL_POINTS_PER_INCH;
 						sprintf (buffer, "%g %g %s BC %s", Ctrl->D.lon + 0.5 * Ctrl->D.width, y0 + d_off, GMT_putfont (GMT, ifont), text);
-						S[TXT] = D[TXT]->table[0]->segment[0];	/* Since there will only be one table with one segment for each set, except for fronts */
-						S[TXT]->record[S[TXT]->n_rows++] = strdup (buffer);
-						// fprintf (stderr, "%s\n", buffer);
-						if (S[TXT]->n_rows == S[TXT]->n_alloc) S[TXT]->record = GMT_memory (GMT, S[TXT]->record, S[TXT]->n_alloc += GMT_SMALL_CHUNK, char *);
+						Ts[TXT] = T[TXT]->table[0]->segment[0];	/* Since there will only be one table with one segment for each set, except for fronts */
+						Ts[TXT]->record[Ts[TXT]->n_rows++] = strdup (buffer);
+						if (debug_print) fprintf (stderr, "%s\n", buffer);
+						if (Ts[TXT]->n_rows == Ts[TXT]->n_alloc) Ts[TXT]->record = GMT_memory (GMT, Ts[TXT]->record, Ts[TXT]->n_alloc += GMT_SMALL_CHUNK, char *);
 						column_number = 0;
 #ifdef DEBUG
 						if (guide) drawbase (GMT, PSL, Ctrl->D.lon, Ctrl->D.lon + Ctrl->D.width, y0);
@@ -718,7 +740,11 @@ int GMT_pslegend (void *V_API, int mode, void *args)
 
 					case 'I':	/* Image record [use GMT_psimage] */
 						sscanf (&line[2], "%s %s %s", image, size, key);
-						PSL_loadimage (PSL, image, &header, &dummy);
+						if (GMT_getdatapath (GMT, image, path, R_OK) == NULL) {
+							GMT_Report (API, GMT_MSG_NORMAL, "Cannot find/open file %s.\n", image);
+							Return (EXIT_FAILURE);
+						}
+						PSL_loadimage (PSL, path, &header, &dummy);
 						PSL_free (dummy);
 						justify = GMT_just_decode (GMT, key, 12);
 						x_off = Ctrl->D.lon;
@@ -734,7 +760,7 @@ int GMT_pslegend (void *V_API, int mode, void *args)
 						break;
 
 					case 'L':	/* Label record */
-						if ((D[TXT] = alloc_if_not_done_already (API, D[TXT], GMT_IS_NONE)) == NULL) return (API->error);
+						if ((T[TXT] = get_textset_pointer (API, T[TXT], GMT_IS_NONE)) == NULL) return (API->error);
 						sscanf (&line[2], "%s %s %s %[^\n]", size, font, key, text);
 						if (size[0] == '-') size[0] = 0;
 						if (font[0] == '-') font[0] = 0;
@@ -747,10 +773,10 @@ int GMT_pslegend (void *V_API, int mode, void *args)
 						x_off = Ctrl->D.lon + (Ctrl->D.width / n_columns) * (column_number%n_columns);
 						x_off += (justify%4 == 1) ? Ctrl->C.dx : ((justify%4 == 3) ? Ctrl->D.width / n_columns - Ctrl->C.dx : 0.5 * Ctrl->D.width / n_columns);
 						sprintf (buffer, "%g %g %s B%s %s", x_off, y0 + d_off, GMT_putfont (GMT, ifont), key, text);
-						S[TXT] = D[TXT]->table[0]->segment[0];	/* Since there will only be one table with one segment for each set, except for fronts */
-						S[TXT]->record[S[TXT]->n_rows++] = strdup (buffer);
-						// fprintf (stderr, "%s\n", buffer);
-						if (S[TXT]->n_rows == S[TXT]->n_alloc) S[TXT]->record = GMT_memory (GMT, S[TXT]->record, S[TXT]->n_alloc += GMT_SMALL_CHUNK, char *);
+						Ts[TXT] = T[TXT]->table[0]->segment[0];	/* Since there will only be one table with one segment for each set, except for fronts */
+						Ts[TXT]->record[Ts[TXT]->n_rows++] = strdup (buffer);
+						if (debug_print) fprintf (stderr, "%s\n", buffer);
+						if (Ts[TXT]->n_rows == Ts[TXT]->n_alloc) Ts[TXT]->record = GMT_memory (GMT, Ts[TXT]->record, Ts[TXT]->n_alloc += GMT_SMALL_CHUNK, char *);
 						column_number++;
 #ifdef DEBUG
 						if (guide) drawbase (GMT, PSL, Ctrl->D.lon, Ctrl->D.lon + Ctrl->D.width, y0);
@@ -856,11 +882,11 @@ int GMT_pslegend (void *V_API, int mode, void *args)
 						if (n == 0 || lspace[0] == '-') sprintf (lspace, "%gi", one_line_spacing);
 						if (n == 0 || tw[0] == '-') sprintf (tw, "%gi", Ctrl->D.width - 2.0 * Ctrl->C.dx);
 						if (n == 0 || jj[0] == '-') sprintf (jj, "j");
-						if ((D[PAR] = alloc_if_not_done_already (API, D[PAR], GMT_IS_NONE)) == NULL) return (API->error);
+						if ((T[PAR] = get_textset_pointer (API, T[PAR], GMT_IS_NONE)) == NULL) return (API->error);
 						sprintf (buffer, "> %s %s %s %s %s %s %s %s", xx, yy, tmp, angle, key, lspace, tw, jj);
-						S[PAR] = D[PAR]->table[0]->segment[0];	/* Since there will only be one table with one segment for each set, except for fronts */
-						S[PAR]->record[S[PAR]->n_rows++] = strdup (buffer);
-						if (S[PAR]->n_rows == S[PAR]->n_alloc) S[PAR]->record = GMT_memory (GMT, S[PAR]->record, S[PAR]->n_alloc += GMT_SMALL_CHUNK, char *);
+						Ts[PAR] = T[PAR]->table[0]->segment[0];	/* Since there will only be one table with one segment for each set, except for fronts */
+						Ts[PAR]->record[Ts[PAR]->n_rows++] = strdup (buffer);
+						if (Ts[PAR]->n_rows == Ts[PAR]->n_alloc) Ts[PAR]->record = GMT_memory (GMT, Ts[PAR]->record, Ts[PAR]->n_alloc += GMT_SMALL_CHUNK, char *);
 						flush_paragraph = true;
 						column_number = 0;
 						break;
@@ -873,17 +899,11 @@ int GMT_pslegend (void *V_API, int mode, void *args)
 						if (column_number%n_columns == 0) y0 -= one_line_spacing;
 						y0 += half_line_spacing;	/* Move to center of box */
 						x_off = x0 + (Ctrl->D.width / n_columns) * (column_number%n_columns);
-						if ((D[SYM] = alloc_if_not_done_already (API, D[SYM], GMT_IS_POINT)) == NULL) return (API->error);
-						S[SYM] = D[SYM]->table[0]->segment[0];	/* Since there will only be one table with one segment for each set, except for fronts */
 						if (symbol[0] == 'f') {	/* Front is different, must plot as a line segment */
-							uint64_t dim[4] = {1, 1, 2, 2};	/* We will a 2-row data set for fronts; allocate just the first time */
 							double length, tlen, gap;
 							int n = sscanf (size, "%[^/]/%[^/]/%s", A, B, C);
-							if (Front == NULL && (Front = GMT_Create_Data (API, GMT_IS_DATASET, GMT_IS_LINE, 0, dim, NULL, NULL, 0, 0, NULL)) == NULL) {
-								GMT_Report (API, GMT_MSG_NORMAL, "Unable to create a Front data set for pslegend\n");
-								return (API->error);
-							}
 							
+							if ((D[FRONT] = get_dataset_pointer (API, D[FRONT], GMT_IS_LINE)) == NULL) return (API->error);
 							if (n == 3) {	/* Got line length, tickgap, and ticklength */
 								length = GMT_to_inch (GMT, A);	/* The length of the line */
 								tlen = GMT_to_inch (GMT, C);	/* The length of the tick */
@@ -903,30 +923,45 @@ int GMT_pslegend (void *V_API, int mode, void *args)
 							else	/* The necessary arguments not supplied, provide reasonable defaults */
 								sprintf (sub, "+l+b");	/* Box to the left of the line is our default front symbol */
 							x = 0.5 * length;
-							F = Front->table[0]->segment[0];	/* Since we only will have one segment */
-							F->coord[GMT_X][0] = x_off + off_ss-x;	F->coord[GMT_Y][0] = y0;
-							F->coord[GMT_X][1] = x_off + off_ss+x;	F->coord[GMT_Y][1] = y0;
-							Front->n_records = F->n_rows = 2;
-							/* Get API ID of the Front object so we can create a file name */
-							if ((object_ID = GMT_Get_ID (API, GMT_IS_DATASET, GMT_IN, Front)) == GMT_NOTSET) {
-								Return (API->error);
-							}
-							if (GMT_Encode_ID (API, string, object_ID) != GMT_OK) {	/* Make filename with embedded object ID */
-								Return (API->error);
-							}
-							sprintf (buffer, "-R0/%g/0/%g -Jx1i -O -K -N -Sf%s/%gi%s %s", GMT->current.proj.rect[XHI], GMT->current.proj.rect[YHI], B, tlen, sub, string);
+							/* Place pen and fill colors in segment header */
+							sprintf (buffer, "> -Sf%s/%gi%s", B, tlen, sub);
 							if (txt_c[0] != '-') {strcat (buffer, " -G"); strcat (buffer, txt_c);}
 							if (txt_d[0] != '-') {strcat (buffer, " -W"); strcat (buffer, txt_d);}
-							// fprintf (stderr, "%s\n", buffer);
+							Ds[FRONT] = D[FRONT]->table[0]->segment[n_fronts];	/* Next segment */
+							Ds[FRONT]->header = strdup (buffer);
+							/* Set begin and end coordinates of the line segment */
+							Ds[FRONT]->coord[GMT_X][0] = x_off + off_ss-x;	Ds[FRONT]->coord[GMT_Y][0] = y0;
+							Ds[FRONT]->coord[GMT_X][1] = x_off + off_ss+x;	Ds[FRONT]->coord[GMT_Y][1] = y0;
+							D[FRONT]->n_records += 2;
+							n_fronts++;
+							if (n_fronts == GMT_SMALL_CHUNK) {
+								GMT_Report (API, GMT_MSG_NORMAL, "Can handle max %d front lines.  Let us know if this is a problem.\n", GMT_SMALL_CHUNK);
+								return (API->error);
+							}
+						}
+						else if (symbol[0] == 'q') {	/* Quoted line is different, must plot as a line segment */
+							double length = GMT_to_inch (GMT, size);	/* The length of the line */;
 							
-							status = GMT_Call_Module (API, "psxy", GMT_MODULE_CMD, buffer);	/* Plot the front */
-							if (status) {
-								GMT_Report (API, GMT_MSG_NORMAL, "GMT_psxy returned error %d.\n", status);
-								Return (EXIT_FAILURE);
+							if ((D[QLINE] = get_dataset_pointer (API, D[QLINE], GMT_IS_LINE)) == NULL) return (API->error);
+							x = 0.5 * length;
+							/* Place pen and fill colors in segment header */
+							sprintf (buffer, "> -S%s", symbol);
+							if (txt_d[0] != '-') {strcat (buffer, " -W"); strcat (buffer, txt_d);}
+							Ds[QLINE] = D[QLINE]->table[0]->segment[n_quoted_lines];	/* Next segment */
+							Ds[QLINE]->header = strdup (buffer);
+							/* Set begin and end coordinates of the line segment */
+							Ds[QLINE]->coord[GMT_X][0] = x_off + off_ss-x;	Ds[QLINE]->coord[GMT_Y][0] = y0;
+							Ds[QLINE]->coord[GMT_X][1] = x_off + off_ss+x;	Ds[QLINE]->coord[GMT_Y][1] = y0;
+							D[QLINE]->n_records += 2;
+							n_quoted_lines++;
+							if (n_quoted_lines == GMT_SMALL_CHUNK) {
+								GMT_Report (API, GMT_MSG_NORMAL, "Can handle max %d quoted lines.  Let us know if this is a problem.\n", GMT_SMALL_CHUNK);
+								return (API->error);
 							}
-							API->io_enabled[GMT_IN] = true;	/* UNDOING SETTING BY psxy */
 						}
 						else {	/* Regular symbols */
+							if ((T[SYM] = get_textset_pointer (API, T[SYM], GMT_IS_POINT)) == NULL) return (API->error);
+							Ts[SYM] = T[SYM]->table[0]->segment[0];	/* Since there will only be one table with one segment for each set, except for fronts */
 							sprintf (sub, "%s", symbol);
 							if (symbol[0] == 'E' || symbol[0] == 'e') {	/* Ellipse */
 								if (strchr (size, ',')) {	/* We got dir,major,minor instead of just size; parse and use */
@@ -958,7 +993,7 @@ int GMT_pslegend (void *V_API, int mode, void *args)
 								/* Because we support both GMT4 and GMT5 vector notations this section is a bit messy */
 								if (strchr (size, ',')) {	/* We got dir,length combined as one argument */
 									sscanf (size, "%[^,],%s", A, B);
-									az1 = GMT_to_inch (GMT, A);
+									az1 = atof (A);
 									x = GMT_to_inch (GMT, B);
 								}
 								else {	/* No dir given, default to horizontal */
@@ -1061,24 +1096,24 @@ int GMT_pslegend (void *V_API, int mode, void *args)
 							sprintf (buffer, ">");
 							strcat (buffer, " -G"); strcat (buffer, txt_c);
 							strcat (buffer, " -W"); strcat (buffer, txt_d);
-							S[SYM]->record[S[SYM]->n_rows++] = strdup (buffer);
-							// fprintf (stderr, "%s\n", buffer);
-							if (S[SYM]->n_rows == S[SYM]->n_alloc) S[SYM]->record = GMT_memory (GMT, S[SYM]->record, S[SYM]->n_alloc += GMT_SMALL_CHUNK, char *);
+							Ts[SYM]->record[Ts[SYM]->n_rows++] = strdup (buffer);
+							if (debug_print) fprintf (stderr, "%s\n", buffer);
+							if (Ts[SYM]->n_rows == Ts[SYM]->n_alloc) Ts[SYM]->record = GMT_memory (GMT, Ts[SYM]->record, Ts[SYM]->n_alloc += GMT_SMALL_CHUNK, char *);
 							sprintf (buffer, "%s %s", sarg, sub);
-							// fprintf (stderr, "%s\n", buffer);
+							if (debug_print) fprintf (stderr, "%s\n", buffer);
 							
-							S[SYM]->record[S[SYM]->n_rows++] = strdup (buffer);
-							if (S[SYM]->n_rows == S[SYM]->n_alloc) S[SYM]->record = GMT_memory (GMT, S[SYM]->record, S[SYM]->n_alloc += GMT_SMALL_CHUNK, char *);
+							Ts[SYM]->record[Ts[SYM]->n_rows++] = strdup (buffer);
+							if (Ts[SYM]->n_rows == Ts[SYM]->n_alloc) Ts[SYM]->record = GMT_memory (GMT, Ts[SYM]->record, Ts[SYM]->n_alloc += GMT_SMALL_CHUNK, char *);
 						}
 						/* Finally, print text; skip when empty */
 						y0 -= half_line_spacing;	/* Go back to bottom of box */
 						if (n_scan == 7) {	/* Place symbol text */
-							if ((D[TXT] = alloc_if_not_done_already (API, D[TXT], GMT_IS_NONE)) == NULL) return (API->error);
-							S[TXT] = D[TXT]->table[0]->segment[0];	/* Since there will only be one table with one segment for each set, except for fronts */
+							if ((T[TXT] = get_textset_pointer (API, T[TXT], GMT_IS_NONE)) == NULL) return (API->error);
+							Ts[TXT] = T[TXT]->table[0]->segment[0];	/* Since there will only be one table with one segment for each set, except for fronts */
 							sprintf (buffer, "%g %g %g,%d,%s BL %s", x_off + off_tt, y0 + d_off, GMT->current.setting.font_annot[0].size, GMT->current.setting.font_annot[0].id, txtcolor, text);
-							// fprintf (stderr, "%s\n", buffer);
-							S[TXT]->record[S[TXT]->n_rows++] = strdup (buffer);
-							if (S[TXT]->n_rows == S[TXT]->n_alloc) S[TXT]->record = GMT_memory (GMT, S[TXT]->record, S[TXT]->n_alloc += GMT_SMALL_CHUNK, char *);
+							if (debug_print) fprintf (stderr, "%s\n", buffer);
+							Ts[TXT]->record[Ts[TXT]->n_rows++] = strdup (buffer);
+							if (Ts[TXT]->n_rows == Ts[TXT]->n_alloc) Ts[TXT]->record = GMT_memory (GMT, Ts[TXT]->record, Ts[TXT]->n_alloc += GMT_SMALL_CHUNK, char *);
 						}
 						column_number++;
 #ifdef DEBUG
@@ -1087,18 +1122,18 @@ int GMT_pslegend (void *V_API, int mode, void *args)
 						break;
 
 					case 'T':	/* paragraph text record */
-						if ((D[PAR] = alloc_if_not_done_already (API, D[PAR], GMT_IS_NONE)) == NULL) return (API->error);
+						if ((T[PAR] = get_textset_pointer (API, T[PAR], GMT_IS_NONE)) == NULL) return (API->error);
 						/* If no previous > record, then use defaults */
-						S[PAR] = D[PAR]->table[0]->segment[0];	/* Since there will only be one table with one segment for each set, except for fronts */
+						Ts[PAR] = T[PAR]->table[0]->segment[0];	/* Since there will only be one table with one segment for each set, except for fronts */
 						if (!flush_paragraph) {
 							d_off = 0.5 * (Ctrl->L.spacing - FONT_HEIGHT_PRIMARY) * GMT->current.setting.font_annot[0].size / PSL_POINTS_PER_INCH;
 							sprintf (buffer, "> %g %g %g,%d,%s 0 TL %gi %gi j", x0, y0 - d_off, GMT->current.setting.font_annot[0].size, GMT->current.setting.font_annot[0].id, txtcolor, one_line_spacing, Ctrl->D.width - 2.0 * Ctrl->C.dx);
-							S[PAR]->record[S[PAR]->n_rows++] = strdup (buffer);
-							if (S[PAR]->n_rows == S[PAR]->n_alloc) S[PAR]->record = GMT_memory (GMT, S[PAR]->record, S[PAR]->n_alloc += GMT_SMALL_CHUNK, char *);
+							Ts[PAR]->record[Ts[PAR]->n_rows++] = strdup (buffer);
+							if (Ts[PAR]->n_rows == Ts[PAR]->n_alloc) Ts[PAR]->record = GMT_memory (GMT, Ts[PAR]->record, Ts[PAR]->n_alloc += GMT_SMALL_CHUNK, char *);
 						}
 						sscanf (&line[2], "%[^\n]", text);
-						S[PAR]->record[S[PAR]->n_rows++] = strdup (text);
-						if (S[PAR]->n_rows == S[PAR]->n_alloc) S[PAR]->record = GMT_memory (GMT, S[PAR]->record, S[PAR]->n_alloc += GMT_SMALL_CHUNK, char *);
+						Ts[PAR]->record[Ts[PAR]->n_rows++] = strdup (text);
+						if (Ts[PAR]->n_rows == Ts[PAR]->n_alloc) Ts[PAR]->record = GMT_memory (GMT, Ts[PAR]->record, Ts[PAR]->n_alloc += GMT_SMALL_CHUNK, char *);
 						flush_paragraph = true;
 						column_number = 0;
 						break;
@@ -1137,68 +1172,105 @@ int GMT_pslegend (void *V_API, int mode, void *args)
 		}
 	}
 
-	if (GMT_Destroy_Data (API, &In) != GMT_OK) {
+	if (GMT_Destroy_Data (API, &In) != GMT_OK) {	/* Remove the main input file from registration */
 		Return (API->error);
 	}
 
 	/* Reset the flag */
 	if (GMT_compat_check (GMT, 4)) GMT->current.setting.io_seg_marker[GMT_IN] = save_EOF;
 
-	if (Front && GMT_Destroy_Data (API, &Front) != GMT_OK) {
-		Return (API->error);
-	}
-	
 	/* Time to plot any symbols, text, and paragraphs we collected in the loop */
 
-	if (S[SYM] && S[SYM]->n_rows) {
-		/* Create option list, register D[SYM] as input source */
-		if ((object_ID = GMT_Get_ID (API, GMT_IS_TEXTSET, GMT_IN, D[SYM])) == GMT_NOTSET) {
+	if (D[FRONT]) {
+		/* Create option list, register D[FRONT] as input source */
+		D[FRONT]->table[0]->n_segments = n_fronts;	/* Set correct number of fronts */
+		if ((object_ID = GMT_Get_ID (API, GMT_IS_DATASET, GMT_IN, D[FRONT])) == GMT_NOTSET) {
+			Return (API->error);
+		}
+		if (GMT_Encode_ID (API, string, object_ID) != GMT_OK) {
+			Return (API->error);	/* Make filename with embedded object ID */
+		}
+		sprintf (buffer, "-R0/%g/0/%g -Jx1i -O -K -N -Sf0.1i %s", GMT->current.proj.rect[XHI], GMT->current.proj.rect[YHI], string);
+		if (debug_print) fprintf (stderr, "%s\n", buffer);
+		if (GMT_Call_Module (API, "psxy", GMT_MODULE_CMD, buffer) != GMT_OK) {	/* Plot the fronts */
+			Return (API->error);
+		}
+		D[FRONT]->table[0]->n_segments = GMT_SMALL_CHUNK;	/* Reset to allocation limit */
+	}
+	if (D[QLINE]) {
+		/* Create option list, register D[QLINE] as input source */
+		D[QLINE]->table[0]->n_segments = n_quoted_lines;	/* Set correct number of lines */
+		if ((object_ID = GMT_Get_ID (API, GMT_IS_DATASET, GMT_IN, D[QLINE])) == GMT_NOTSET) {
+			Return (API->error);
+		}
+		if (GMT_Encode_ID (API, string, object_ID) != GMT_OK) {
+			Return (API->error);	/* Make filename with embedded object ID */
+		}
+		sprintf (buffer, "-R0/%g/0/%g -Jx1i -O -K -N -Sqn1 %s", GMT->current.proj.rect[XHI], GMT->current.proj.rect[YHI], string);
+		if (debug_print) fprintf (stderr, "%s\n", buffer);
+		if (GMT_Call_Module (API, "psxy", GMT_MODULE_CMD, buffer) != GMT_OK) {	/* Plot the fronts */
+			Return (API->error);
+		}
+		D[QLINE]->table[0]->n_segments = GMT_SMALL_CHUNK;	/* Reset to allocation limit */
+	}
+	if (T[SYM]) {
+		/* Create option list, register T[SYM] as input source */
+		if ((object_ID = GMT_Get_ID (API, GMT_IS_TEXTSET, GMT_IN, T[SYM])) == GMT_NOTSET) {
 			Return (API->error);
 		}
 		if (GMT_Encode_ID (API, string, object_ID) != GMT_OK) {
 			Return (API->error);	/* Make filename with embedded object ID */
 		}
 		sprintf (buffer, "-R0/%g/0/%g -Jx1i -O -K -N -S %s", GMT->current.proj.rect[XHI], GMT->current.proj.rect[YHI], string);
-		// fprintf (stderr, "%s", buffer);
-		if (GMT_Call_Module (API, "psxy", GMT_MODULE_CMD, buffer) != GMT_OK) {
-			Return (API->error);	/* Plot the symbols */
+		if (debug_print) fprintf (stderr, "%s\n", buffer);
+		if (GMT_Call_Module (API, "psxy", GMT_MODULE_CMD, buffer) != GMT_OK) {	/* Plot the symbols */
+			Return (API->error);
 		}
 	}
-	if (S[TXT] && S[TXT]->n_rows) {
-		/* Create option list, register D[TXT] as input source */
-		if ((object_ID = GMT_Get_ID (API, GMT_IS_TEXTSET, GMT_IN, D[TXT])) == GMT_NOTSET) {
+	if (T[TXT]) {
+		/* Create option list, register T[TXT] as input source */
+		if ((object_ID = GMT_Get_ID (API, GMT_IS_TEXTSET, GMT_IN, T[TXT])) == GMT_NOTSET) {
 			Return (API->error);
 		}
 		if (GMT_Encode_ID (API, string, object_ID) != GMT_OK) {
 			Return (API->error);	/* Make filename with embedded object ID */
 		}
 		sprintf (buffer, "-R0/%g/0/%g -Jx1i -O -K -N -F+f+j %s", GMT->current.proj.rect[XHI], GMT->current.proj.rect[YHI], string);
-		if (GMT_Call_Module (API, "pstext", GMT_MODULE_CMD, buffer) != GMT_OK) {
-			Return (API->error);	/* Plot the symbols */
+		if (debug_print) fprintf (stderr, "%s\n", buffer);
+		if (GMT_Call_Module (API, "pstext", GMT_MODULE_CMD, buffer) != GMT_OK) {	/* Plot the symbol labels */
+			Return (API->error);
 		}
 	}
-	if (S[PAR] && S[PAR]->n_rows) {
-		/* Create option list, register D[PAR] as input source */
-		if ((object_ID = GMT_Get_ID (API, GMT_IS_TEXTSET, GMT_IN, D[PAR])) == GMT_NOTSET) {
+	if (T[PAR]) {
+		/* Create option list, register T[PAR] as input source */
+		if ((object_ID = GMT_Get_ID (API, GMT_IS_TEXTSET, GMT_IN, T[PAR])) == GMT_NOTSET) {
 			Return (API->error);
 		}
 		if (GMT_Encode_ID (API, string, object_ID) != GMT_OK) {
 			Return (API->error);	/* Make filename with embedded object ID */
 		}
 		sprintf (buffer, "-R0/%g/0/%g -Jx1i -O -K -N -M -F+f+a+j %s", GMT->current.proj.rect[XHI], GMT->current.proj.rect[YHI], string);
-		if (GMT_Call_Module (API, "pstext", GMT_MODULE_CMD, buffer) != GMT_OK) Return (API->error);	/* Plot the symbols */
-	}
-
-	for (id = 0; id < 3; id++) {
-		if (D[id] && GMT_Destroy_Data (API, &D[id]) != GMT_OK) {
+		if (debug_print) fprintf (stderr, "%s\n", buffer);
+		if (GMT_Call_Module (API, "pstext", GMT_MODULE_CMD, buffer) != GMT_OK) {	/* Plot paragraphs */
 			Return (API->error);
 		}
 	}
+
 	PSL_setorigin (PSL, -x_orig, -y_orig, 0.0, PSL_INV);	/* Reset */
 
 	GMT_map_basemap (GMT);
 	GMT_plotend (GMT);
 
+	if (D[FRONT] && GMT_Destroy_Data (API, &D[FRONT]) != GMT_OK) {
+		Return (API->error);
+	}
+	
+	for (id = 0; id < N_TXT; id++) {
+		if (T[id] && GMT_Destroy_Data (API, &T[id]) != GMT_OK) {
+			Return (API->error);
+		}
+	}
+	
 	GMT_Report (API, GMT_MSG_VERBOSE, "Done\n");
 
 	Return (GMT_OK);
